Attribute VB_Name = "modCalcComb"
'==============================================================
' Descrizione.....: Routines di calcolo combinatorio.
' Nome dei Files..: CalcComb.bas
' Data............: 19/12/1999
' Aggiornamento:..: 4/2/2001  (aggiunte le Permutazioni
'                              con ripetizione).
' Aggiornamento:..: 17/1/2002 (aggiunta la gestione degli
'                              errori di Overflow).
' Versione........: 1.2 a 32 bits.
' Sistema.........: VB6 sotto Windows NT.
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
' DownLoads a.....: http://www.flanguasco.org
'                   http://members.xoom.virgilio.it/flanguasco/
'==============================================================
'   Comprende:
'
'    CombinazioniS:     Trova le combinazioni semplici di N oggetti
'                       della classe K.
'
'    CombinazioniS_RR:  Trova le combinazioni semplici di N oggetti
'                       della classe K, una per volta. E' utile quando
'                       il numero delle combinazioni da trovare e'
'                       troppo grande per essere ospitato in una
'                       matrice (e.g. 90 oggetti presi a gruppi di 6 ...).
'
'    DisposizioniS:     Trova le disposizioni semplici di N oggetti
'                       della classe K. Se K = N trova le permutazioni
'                       di N oggetti.
'
'    PermutazioniR:     Trova le permutazioni con ripetizione di N oggetti
'                       della classe K.
'
'    PermutazioniR_RR:  Trova le permutazioni con ripetizione di N oggetti
'                       della classe K, una per volta. E' utile quando
'                       il numero delle permutazioni da trovare e'
'                       troppo grande per essere ospitato in una
'                       matrice (e.g. 3 oggetti presi a gruppi di 13 ...).
'
'    Cnk:               Ritorna il numero delle combinazioni semplici di N
'                       oggetti presi a gruppi di K.
'
'    Wnk:               Ritorna il numero delle combinazioni con ripetizione di N
'                       oggetti presi a gruppi di K.
'
'    Dnk:               Ritorna il numero delle disposizioni semplici di N
'                       oggetti presi a gruppi di K.
'
'    Vnk:               Ritorna il numero delle permutazioni con ripetizione di N
'                       oggetti presi a gruppi di K.
'
'    Prodotto:          Ritorna il prodotto dei numeri da N1 a N2.
'                       Se N1 manca, ritorna il fattoriale di N2.
'
'   Nota:   Tutti i vettori e le matrici di queste
'           routines iniziano dall' indice 1.
'
Option Explicit
'
Private Const Overflow& = 6 ' Codice dell' errore di "overflow".
Public Function CombinazioniS_RR(ByVal N&, ByVal K&, CombiS&() _
    , ByRef NR&, ByRef Ultima As Boolean) As Boolean
'
'   Ritorna, nel vettore CombiS(1 to K), una nuova riga di
'   combinazioni semplici di N oggetti della classe K, con
'   N >= K e K > 0.
'   Se viene passato NR = 0 inizia un nuovo ciclo di ricerca
'   delle combinazioni volute e successivamente ritorna NR con
'   il numero, progressivo, della combinazione trovata.
'   Quando viene emessa l' ultima combinazione possibile,
'   ritorna con Ultima = True e NR = Cnk(N, K).
'   AVVERTENZA: non cambiare, nel codice chiamante e durante un
'               ciclo di ricerca, i valori di N, K, NR e CombiS().
'
    Dim C&, P&
    Static NRMax&
'
    On Error GoTo CombinazioniS_RR_ERR
'
    If NR = 0 Then
        ' Prima riga:
        NR = 1
        NRMax = Cnk(N, K)   ' Numero delle combinazioni possibili.
        ReDim CombiS(1 To K)
'
        For C = 1 To K
            CombiS(C) = C
        Next C
'
    ElseIf NR < NRMax Then
        ' Righe successive:
        NR = NR + 1
'
        P = K
        Do Until CombiS(P) < N - K + P
            P = P - 1
        Loop
'
        CombiS(P) = CombiS(P) + 1
'
        For C = P + 1 To K
            CombiS(C) = CombiS(C - 1) + 1
        Next C
    End If
'
    Ultima = (NR = NRMax)
'
'
CombinazioniS_RR_ERR:
    CombinazioniS_RR = (Err = 0)
    If (Err <> 0) Then
        Dim M$
        M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
        MsgBox M$, vbCritical, " Function CombinazioniS_RR"
    End If
'
'
'
End Function
Public Function Wnk(ByVal N&, ByVal K&) As Long
'
'   Ritorna il numero delle combinazioni con ripetizione
'   di N oggetti presi a gruppi di K:
'
    On Error GoTo Wnk_ERR
'
    Wnk = CLng(Prodotto(N + K - 1, N) / Prodotto(K))
    Exit Function
'
'
Wnk_ERR:
    Dim M$
    Wnk = 0
    M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
    If (Err = Overflow) Then
        M$ = M$ & "Il numero di combinazioni con ripetizione" & vbNewLine
        M$ = M$ & "e' troppo grande." & vbNewLine
    End If
    MsgBox M$, vbCritical, " Function Wnk"
'
'
'
End Function
Public Function Cnk(ByVal N&, ByVal K&) As Long
'
'   Ritorna il numero delle combinazioni semplici
'   di N oggetti presi a gruppi di K. Se K = 0 e',
'   per definizione, Cnk(N, 0) = 1:
'
    On Error GoTo Cnk_ERR
'
    Cnk = CLng(Prodotto(N, N - K + 1) / Prodotto(K))
    Exit Function
'
'
Cnk_ERR:
    Dim M$
    Cnk = 0
    M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
    If (Err = Overflow) Then
        M$ = M$ & "Il numero di combinazioni e' troppo grande." & vbNewLine
    End If
    MsgBox M$, vbCritical, " Function Cnk"
'
'
'
End Function
Public Function CombinazioniS(ByVal N&, ByVal K&, CombiS&()) As Boolean
'
'   Ritorna, nella matrice CombiS(Cnk(N, K), K) con N >= K
'   e K > 0, le combinazioni semplici di N oggetti della classe K:
'
    Dim R&, NR&, C&, P&
'
    On Error GoTo CombinazioniS_ERR
'
    NR = Cnk(N, K)  ' Numero delle righe.
    ReDim CombiS(1 To NR, 1 To K)
'
    ' Scrivo la prima riga:
    For C = 1 To K
        CombiS(1, C) = C
    Next C
'
    ' Righe rimanenti:
    For R = 2 To NR
'
        P = K
        Do Until CombiS(R - 1, P) < N - K + P
            P = P - 1
        Loop
'
        For C = 1 To P - 1
            CombiS(R, C) = CombiS(R - 1, C)
        Next C
'
        CombiS(R, P) = CombiS(R - 1, P) + 1
'
        For C = P + 1 To K
            CombiS(R, C) = CombiS(R, C - 1) + 1
        Next C
'
    Next R
'
'
CombinazioniS_ERR:
    CombinazioniS = (Err = 0)
    If (Err <> 0) Then
        Dim M$
        M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
        MsgBox M$, vbCritical, " Function CombinazioniS"
    End If
'
'
'
End Function
Public Function Prodotto(ByVal N2&, Optional ByVal N1& = 2) As Double
'
'   Ritorna il prodotto dei numeri, consecutivi, interi e positivi,
'   da N1 a N2 (0 < N1 <= N2). Se N1 > N2 ritorna 1.
'   Se N1 manca, ritorna il Fattoriale di N2; in questo caso puo'
'   anche essere N2 = 0 perche', per definizione, e' 0! = 1:
'
    Dim Pr#, I&
'
    Pr = 1#
    For I = N1 To N2
        Pr = Pr * CDbl(I)
    Next I
'
    Prodotto = Pr
'
'
'
End Function
Public Function DisposizioniS_V(ByVal N&, ByVal K&, DispoS&()) As Boolean
'
'   Ritorna, nella matrice DispoS(Dnk(N, K), K) con N >= K
'   e K > 0, le disposizioni semplici di N oggetti della classe K.
'   Se K = N calcola le permutazioni di N oggetti.
'
'   Questa versione sacrifica la compattezza del codice alla velocita'
'   di esecuzione:
'
    Dim R&, NR&, C&, cr&, P&, I&, J&, L&
    Dim NG1&, DG1&, NG2&, DG2&  ' Numero e dimensioni dei sottogruppi.
'
    On Error GoTo DisposizioniS_V_ERR
'
    NR = Dnk(N, K)  ' Numero delle righe.
    ReDim DispoS(1 To NR, 1 To K)
'
    ' Scrivo la prima colonna:
    R = 1
    P = 1
    NG2 = N
    DG2 = NR / NG2
    For J = 1 To NG2
        For I = 1 To DG2
            DispoS(R, 1) = P
            R = R + 1
        Next I
        P = P + 1
    Next J
'
    ' Colonne rimanenti:
    For C = 2 To K
        R = 1
        NG1 = NR / DG2
        DG1 = DG2
        NG2 = N - C + 1
        DG2 = DG1 / NG2
        For L = 1 To NG1
            P = DispoS(R, C - 1) + 1
            For J = 1 To NG2
                Do  ' Convalida P o cerca un altro valore.
                    If P > N Then P = 1
                    For cr = 1 To C - 1
                        If P = DispoS(R, cr) Then
                            P = P + 1
                            Exit For
                        End If
                    Next cr
                Loop Until cr = C
'
                For I = 1 To DG2
                    DispoS(R, C) = P
                    R = R + 1
                Next I
                P = P + 1
            Next J
        Next L
    Next C
'
'
DisposizioniS_V_ERR:
    DisposizioniS_V = (Err = 0)
    If (Err <> 0) Then
        Dim M$
        M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
        MsgBox M$, vbCritical, " Function DisposizioniS_V"
    End If
'
'
'
End Function
Public Function DisposizioniS(ByVal N&, ByVal K&, DispoS&()) As Boolean
'
'   Ritorna, nella matrice DispoS(Dnk(N, K), K) con N >= K
'   e K > 0, le disposizioni semplici di N oggetti della classe K.
'   Se K = N calcola le permutazioni di N oggetti:
'
    Dim R&, NR&, C&, cr&, P&, I&, J&, L&
    Dim NG1&, DG1&, NG2&, DG2&  ' Numero e dimensioni dei sottogruppi.
'
    On Error GoTo DisposizioniS_ERR
'
    NR = Dnk(N, K)  ' Numero delle righe.
    ReDim DispoS(1 To NR, 1 To K)
'
    DG2 = NR
    For C = 1 To K
        R = 1
        NG1 = NR / DG2
        DG1 = DG2
        NG2 = N - C + 1
        DG2 = DG1 / NG2
        For L = 1 To NG1
            P = 1
            For J = 1 To NG2
                GoSub CercaP
                For I = 1 To DG2
                    DispoS(R, C) = P
                    R = R + 1
                Next I
                P = P + 1
            Next J
        Next L
    Next C
'
'
DisposizioniS_ERR:
    DisposizioniS = (Err = 0)
    If (Err <> 0) Then
        Dim M$
        M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
        MsgBox M$, vbCritical, " Function DisposizioniS"
    End If
'
    Exit Function
'
'
CercaP:
    ' Verifica la validita' di P per la posizione corrente
    ' (R, C) ed eventualmente ricerca, in modo ricorsivo,
    ' un valore alternativo:
    If P > N Then P = 1
    For cr = 1 To C - 1
        If P = DispoS(R, cr) Then
            P = P + 1
            GoSub CercaP
        End If
    Next cr
    Return
'
'
'
End Function
Public Function PermutazioniR_RR(ByVal N&, ByVal K&, PermuR&() _
    , ByRef NR&, ByRef Ultima As Boolean) As Boolean
'
'   Ritorna, nel vettore PermuR(1 to K), una nuova riga di
'   permutazioni con ripetizione di N oggetti della classe K,
'   con N > 0 e K > 0.
'   Se viene passato NR = 0 inizia un nuovo ciclo di ricerca
'   delle permutazioni volute e successivamente ritorna NR con
'   il numero, progressivo, della permutazione trovata.
'   Quando viene emessa l' ultima permutazione possibile,
'   ritorna con Ultima = True e NR = Vnk(N, K).
'   AVVERTENZA: non cambiare, nel codice chiamante e durante un
'               ciclo di ricerca, i valori di N, K, NR e PermuR().
'
    Dim J&
    Static NRMax&
'
    On Error GoTo PermutazioniR_RR_ERR
'
    If NR = 0 Then
        NR = 1
        NRMax = Vnk(N, K)   ' Numero delle righe.
        ReDim PermuR(1 To K)
    '
        ' Prepara la prima riga
        ' con tutti 1:
        For J = 1 To K
            PermuR(J) = 1
        Next J
'
    ElseIf NR < NRMax Then
        ' Righe successive:
        NR = NR + 1
'
        ' Incrementa, progressivamente,
        ' i valori della riga, fino a
        ' trovare un' assenza di riporto:
        For J = 1 To K
            PermuR(J) = PermuR(J) + 1
            If PermuR(J) > N Then
                PermuR(J) = 1
            Else
                Exit For
            End If
        Next J
    End If
'
    Ultima = (NR = NRMax)
'
'
PermutazioniR_RR_ERR:
    PermutazioniR_RR = (Err = 0)
    If (Err <> 0) Then
        Dim M$
        M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
        MsgBox M$, vbCritical, " Function PermutazioniR_RR"
    End If
'
'
'
End Function
Public Function PermutazioniR(ByVal N&, ByVal K&, PermuR&()) As Boolean
'
'   Ritorna, nella matrice PermuR(Vnk(N, K), K) con N > 0 e K > 0,
'   le permutazioni con ripetizione di N oggetti della classe K.
'
    Dim I&, J&, NR&
'
    On Error GoTo PermutazioniR_ERR
'
    NR = Vnk(N, K)  ' Numero delle righe.
    ReDim PermuR(1 To NR, 1 To K)
'
    ' Prepara la prima riga
    ' con tutti 1:
    For J = 1 To K
        PermuR(1, J) = 1
    Next J
'
    For I = 2 To NR
        ' Ricopia la riga precedente:
        For J = 1 To K
            PermuR(I, J) = PermuR(I - 1, J)
        Next J
'
        ' Incrementa, progressivamente,
        ' i valori della riga, fino a
        ' trovare un' assenza di riporto:
        For J = 1 To K
            PermuR(I, J) = PermuR(I, J) + 1
            If PermuR(I, J) > N Then
                PermuR(I, J) = 1
            Else
                Exit For
            End If
        Next J
    Next I
'
'
PermutazioniR_ERR:
    PermutazioniR = (Err = 0)
    If (Err <> 0) Then
        Dim M$
        M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
        MsgBox M$, vbCritical, " Function PermutazioniR"
    End If
'
'
'
End Function
Public Function Vnk(ByVal N&, ByVal K&) As Long
'
'   Ritorna il numero delle permutazioni con ripetizione
'   di N oggetti presi a gruppi di K:
'
    On Error GoTo Vnk_ERR
'
    Vnk = N ^ K
    Exit Function
'
'
Vnk_ERR:
    Dim M$
    Vnk = 0
    M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
    If (Err = Overflow) Then
        M$ = M$ & "Il numero di permutazioni con ripetizione" & vbNewLine
        M$ = M$ & "e' troppo grande." & vbNewLine
    End If
    MsgBox M$, vbCritical, " Function Vnk"
'
'
'
End Function
Public Function Dnk(ByVal N&, ByVal K&) As Long
'
'   Ritorna il numero delle disposizioni semplici
'   di N oggetti presi a gruppi di K:
'
    On Error GoTo Dnk_ERR
'
    Dnk = CLng(Prodotto(N, N - K + 1))
    Exit Function
'
'
Dnk_ERR:
    Dim M$
    Dnk = 0
    M$ = "Err. N " & Err.Number & "  " & Err.Description & vbNewLine
    If (Err = Overflow) Then
        M$ = M$ & "Il numero di disposizioni e' troppo grande." & vbNewLine
    End If
    MsgBox M$, vbCritical, " Function Dnk"
'
'
'
End Function
